@@ -357,7 +357,7 @@
 
         for (i in seq(length(labelist$files))) {
             if (pathtofiles) {
-                obj <- getMetadata(
+                obj <- getCodebook(
                     file.path(
                         labelist$completePath,
                         labelist$files[i]
@@ -367,14 +367,14 @@
                     # save = saveFile
                 )
 
-                # --- TODO getMetadata() doesn't embed the data anymore...
-                # do I really need to?
-                tmp <- extractData(obj)
-                # ---
+                ### --- TODO getCodebook() doesn't embed the data anymore...
+                ## do I really need to?
+                # tmp <- extractData(obj)
 
-                if (!is.null(tmp)) {
-                    csv <- tmp
-                }
+                # if (!is.null(tmp)) {
+                #     csv <- tmp
+                # }
+                ### ---
             }
             else {
                 aa <- ls()
