@@ -18,6 +18,9 @@
 #'
 #' @export
 `updateCodebook` <- function(xmlfile, with, ...) {
+
+    DDIC <- get("DDIC", envir = cacheEnv)
+
     dots <- list(...)
     indent <- checkDots(dots$indent, default = 2)
     enter <- checkDots(
@@ -54,7 +57,7 @@
         which(grepl(paste0("</", s), xml))
     })
     names(ends) <- sections
-    
+
     for (s in sections) {
         if (length(starts[[s]]) > 0 && length(ends[[s]]) == 0) {
             ends[[s]] <- starts[[s]]
@@ -65,7 +68,7 @@
 
     tmp <- tempdir()
 
-    monolang <- codeBook$.extra$monolang
+    monolang <- with$.extra$monolang
     if (is.null(monolang)) monolang <- FALSE
 
     codeBook <- removeExtra(changeXMLang(with, remove = monolang))
@@ -83,7 +86,7 @@
                 return(part[seq(3, length(part) - 1)])
             }
         ))
-        
+
         start <- starts[[child]]
         end <- ends[[child]]
 
@@ -104,4 +107,4 @@
     }
 
     writeLines(xml, con = xmlfile)
-}
+}
\ No newline at end of file
